
start                   equ   32768

                        org   start

                        jp    prepare_state

INK                     macro inkcol
                        ld    a,16
                        rst   16
                        ld    a,inkcol
                        rst   16
                        endm

main                    proc

                        ; breakpoint here to make test snaps

                        ; *** preserve register state first ***
                        ; =====================================
                        ld    (register_SP),sp

                        ld    sp,register_SP
                        push  af
                        push  bc

                        ld    a,i
                        ld    b,a
                        ld    a,r
                        sub   10          ; adjust prepared R register value
                        ld    c,a
                        push  bc          ; register IR

                        push  iy
                        push  ix

                        push  de
                        push  hl

                        exx
                        ex    af,af'
                        push  af
                        push  bc
                        push  de
                        push  hl


                        ; restore sanity before continuing
                        ; ================================
                        ld    sp,our_stack
                        ld    a,#3f
                        ld    i,a
                        ld    iy,23610


                        ; test 48K memory before clearing screen
                        ; ======================================
                        ld    ix,compare_mem
                        call  fill_comp_48k_mem
                        ld    a,1
                        jr    z,_mem_pass
                        xor   a
            _mem_pass   ld    (mem_test_passed),a
                        ld    (mem_test_fail_addr),hl ; hl holds first failed mem address if test failed


                        ; now we can clear the screen ready for print results
                        ; ===================================================
                        ld    a,7
                        call  8859
                        ld    a,56
                        ld    (23658),a
                        call  3435
                        ld    a,2
                        call  #1601

                        ; start to process register values
                        ; ================================
                        call  print_text
                        db    22,0,0,"Regular set",13,255
                        ld    hl,(register_HL)
                        ld    de,(prepared_HL)
                        ld    bc,+("H"<<8)|"L"
                        call  compare_register_16

                        ld    hl,(register_DE)
                        ld    de,(prepared_DE)
                        ld    bc,+("D"<<8)|"E"
                        call  compare_register_16

                        ld    hl,(register_BC)
                        ld    de,(prepared_BC)
                        ld    bc,+("B"<<8)|"C"
                        call  compare_register_16

                        ld    hl,(register_AF)
                        ld    de,(prepared_AF)
                        ld    bc,+("A"<<8)|"F"
                        call  compare_register_16

                        call  print_text
                        db    22,4,0,"Alternate set",13,255
                        ld    hl,(register_hl)
                        ld    de,(prepared_hl)
                        ld    bc,+("H"<<8)|"L"
                        call  compare_register_16

                        ld    hl,(register_de)
                        ld    de,(prepared_de)
                        ld    bc,+("D"<<8)|"E"
                        call  compare_register_16

                        ld    hl,(register_bc)
                        ld    de,(prepared_bc)
                        ld    bc,+("B"<<8)|"C"
                        call  compare_register_16

                        ld    hl,(register_af)
                        ld    de,(prepared_af)
                        ld    bc,+("A"<<8)|"F"
                        call  compare_register_16

                        call  print_text
                        db    22,8,0,"Index set",13,255
                        ld    hl,(register_IX)
                        ld    de,(prepared_IX)
                        ld    bc,+("I"<<8)|"X"
                        call  compare_register_16

                        ld    hl,(register_IY)
                        ld    de,(prepared_IY)
                        ld    bc,+("I"<<8)|"Y"
                        call  compare_register_16

                        call  print_text
                        db    22,11,0,"Last feckers",13,255
                        ld    hl,(register_SP)
                        ld    de,(prepared_SP)
                        ld    bc,+("S"<<8)|"P"
                        call  compare_register_16

                        ld    hl,(register_IR)
                        ld    de,(prepared_IR)
                        ld    bc,+("I"<<8)|"R"
                        call  compare_register_16


                        ; report 48K memory test result
                        ; =============================
                        call  print_text
                        db    22,14,0,"48K memory test ",255

                        ld    a,(mem_test_passed)
                        or    a
                        jr    nz,_memtest_pass

      _memtest_fail     INK   2
                        call  print_text
                        db    "failed at #",255
                        ld    hl,(mem_test_fail_addr) ; hl holds first failed mem address
                        call  print_hex_16
                        INK   0
                        jr    _memtest_end

      _memtest_pass     call  print_text
                        db    "passed",255
      _memtest_end 


                        ; end of current tests
                        ; ====================
                        ld    a,13
                        rst   16

                        ; exit to BASIC
                        ld    sp,(basic_sp)
                        ld    hl,#2758
                        exx
                        ei
                        ret
                        endp


compare_register_16     proc
                        INK   0
                        ld    a,b               ; print 16 bit register name
                        rst   16
                        ld    a,c
                        rst   16
                        ld    a,":"
                        rst   16
                        ld    a," "
                        rst   16

                        or    a
                        sbc   hl,de
                        add   hl,de
                        push  af
                        jr    z,_not_ink_2

                        INK   2

            _not_ink_2  call  print_hex_16      ; print HL value

                        INK   0
                        ld    a," "
                        rst   16

                        pop   af
                        jr    z,_passed

            _failed     ld    a,"("
                        rst   16

                        ex    de,hl
                        call  print_hex_16      ; print expected HL value

                        call  print_text
                        db    ")",6,255
                        ret

            _passed     call  print_text
                        db    "pass",6,255
                        ret
                        endp



                        ld    bc,5000
                        ld    e,2

                        xor   a
                        ld    r,a

      _main_loop        ld    a,r
                        cp    e
                        jp    nz,_pfft          ; nz = failed

                        ld    a,e
                        add   a,13
                        and   127
                        ld    e,a

                        ld    d,0
            _delay1     dec   d
                        jr    nz,_delay1

                        dec   bc
                        ld    a,b
                        or    c
                        jr    nz,_main_loop     ; z = passed

      _passed           ei
                        call  print_text
                        db    "passed",13,255
                        ret

      _pfft             ld    h,a   ; was R
                        ld    l,e   ; was E
                        push  hl

                        ei
                        call  print_text
                        db    16,2,"pfft!",16,0,13,255

                        pop   hl
                        call  print_text
                        db    "Expected R=",255
                        ld    a,l   ; was E
                        call  print_hex_8

                        call  print_text
                        db    ", got R=",255
                        ld    a,h   ; was R
                        call  print_hex_8
                        ld    a,13
                        rst   16
                        ret

print_text              proc
                        ld    (_old_DE+1),de
                        pop   de
      _loop             ld    a,(de)
                        inc   de
                        cp    255
                        jr    z,_exit
                        rst   16
                        jr    _loop

      _exit             push  de
            _old_DE     ld    de,0
                        ret
                        endp

print_hex_16            proc
                        ld    a,h
                        call  print_hex_8
                        ld    a,l
                        call  print_hex_8
                        ret
                        endp

print_hex_8             proc
                        push  af
                        rra
                        rra
                        rra
                        rra
                        call  print_hex_char
                        pop   af
                        call  print_hex_char
                        ret
                        endp

print_hex_char          proc
                        and   15
                        cp    10
                        sbc   a,#69
                        daa
                        rst   16
                        ret
                        endp

fill_comp_48k_mem       proc
                        ld    hl,16384+64
                        ld    bc,6144-64-64     ; omit first and last 64 bytes of display memory
                        call  _jmp_fnc
                        ret   nz

                        ld    hl,25000
                        ld    bc,7000
                        call  _jmp_fnc
                        ret   nz

                        ld    hl,code_end
                        ld    bc,65536-code_end
                        call  _jmp_fnc
                        ret

      _jmp_fnc          jp    (ix)
                        endp

fill_mem                proc
                        ld    a,128
            _loop       ld    (hl),a
                        inc   a
                        inc   hl
                        dec   c
                        jr    nz,_loop
                        dec   b
                        jp    p,_loop
                        cp    a           ; set z = pass as if compare test passed
                        ret
                        endp

compare_mem             proc
                        ld    a,128
            _loop       cp    (hl)
                        ret   nz          ; nz = fail
                        inc   a
                        inc   hl
                        dec   c
                        jr    nz,_loop
                        dec   b
                        jp    p,_loop
                        cp    a           ; z = pass
                        ret
                        endp

mem_test_passed         db    0
mem_test_fail_addr      dw    0

basic_sp                dw    0
                        ds    128
our_stack               ; marks top of our working stack area


                        ; register state stack
; alternate set
prepared_hl             dw    #0123
prepared_de             dw    #1234
prepared_bc             dw    #2345
prepared_af             dw    #3456

; regular set
prepared_HL             dw    #4567
prepared_DE             dw    #5678

prepared_IX             dw    #7890
prepared_IY             dw    #8901

prepared_IR             dw    #9012

prepared_BC             dw    #6789
prepared_AF             dw    #a123

prepared_SP             dw    0


; alternate set
register_hl             dw    0
register_de             dw    0
register_bc             dw    0
register_af             dw    0

; regular set
register_HL             dw    0
register_DE             dw    0

register_IX             dw    0
register_IY             dw    0

register_IR             dw    0

register_BC             dw    0
register_AF             dw    0

register_SP             dw    0


prepare_state           proc

                        di
                        ld    (basic_sp),sp

                        ld    a,201       ; RET
                        ld    (start),a   ; test will only run once

                        ; prepare 48K memory
                        ld    ix,fill_mem
                        call  fill_comp_48k_mem


                        ; prepare state
                        im    1

                        ld    sp,prepared_hl
                        pop   hl
                        pop   de
                        pop   bc
                        pop   af
                        exx
                        ex    af,af'

                        pop   hl
                        pop   de

                        pop   ix
                        pop   iy

                        pop   bc          ; prepared_IR
                        ld    a,b
                        ld    i,a
                        ld    a,c
                        sub   6           ; adjust prepared R register value
                        ld    r,a

                        pop   bc
                        pop   af


                        ; prepare to JP to main
                        ld    sp,our_stack
                        ld    (prepared_SP),sp

                        jp    main
                        endp

code_end
                        end   start

